/////////////////////////////////////////////////////////////////////////////
// GliD3D_Render.cpp : No comment
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Directive

#include "StdHdr.h"
#pragma hdrstop
#include "GliD3D.h"

/////////////////////////////////////////////////////////////////////////////
// Defines / Variables

#define GLID3D_ANTIALIAS_ENABLE 

#define GLID3D_ANTIALIAS_DISABLE 


static WORD idx[4096];
static FLOAT f128 = 128.0f;
static FLOAT f256 = 256.0f;
static FLOAT f4096 = 4096.0f;
static FLOAT f65535 = 65535.0f;
static FLOAT f65536 = 65536.0f;

/////////////////////////////////////////////////////////////////////////////
// Prototypes

/////////////////////////////////////////////////////////////////////////////
// Implements

inline VOID
grsubSetVertexFromGrVertex(VERTEX *ptlv, const GrVertex* pv)
{
	int ah, aw;

	INT nTmu;

//	nTmu = grsubGetTmuIndex(tmu);
	nTmu = 0;
grsubAspectToInt(&aw, &ah, g_gr_tmu[nTmu].texture_source_info.aspectRatio);

	if (pv->x < f4096)
		ptlv->cord.x = pv->x;
	else
		ptlv->cord.x = pv->x - (FLOAT)(((DWORD)pv->x) & 0xfffff000);

	if (g_gr_state.origin_location == GR_ORIGIN_UPPER_LEFT) {
		if (pv->y < f4096)
			ptlv->cord.y = pv->y;
		else
			ptlv->cord.y = pv->y - (FLOAT)(((DWORD)pv->y) & 0xfffff000);

	} else {
		if (pv->y < f4096)
			ptlv->cord.y = (FLOAT)g_gr_nHeight - pv->y;
		else
			ptlv->cord.y = (FLOAT)g_gr_nHeight - (FLOAT)(((DWORD)pv->y) & 0xfffff000);

	}

	ptlv->cord.w = pv->oow;
	if(g_gr_bUseZInverse)
	{

		if(g_gr_bPrecisionFix)
		{
			if(ptlv->cord.w > 0.1f)
				ptlv->cord.w = 0.1f;

		}

		ptlv->cord.w = 1.0/ptlv->cord.w;
		ptlv->cord.z = 1.0f;
	}
	else
	{

		ptlv->cord.w = 1.0/pv->oow;

		ptlv->cord.z = -ptlv->cord.w * ptlv->cord.w;
	}

	ptlv->cord.x *= ptlv->cord.w;
	ptlv->cord.y *= ptlv->cord.w;

	

	switch (g_gr_state.color_combine_local) {
	case GR_COMBINE_LOCAL_ITERATED:
		switch (g_gr_state.alpha_combine_local) {
		case GR_COMBINE_LOCAL_ITERATED:
			ptlv->color = ((DWORD)pv->r << 16) | ((DWORD)pv->g <<  8) | ((DWORD)pv->b <<  0) | ((DWORD)pv->a << 24);
			break;
		case GR_COMBINE_LOCAL_CONSTANT:
			ptlv->color = ((DWORD)pv->r << 16) | ((DWORD)pv->g <<  8) | ((DWORD)pv->b <<  0) | (g_gr_state.constant_color_value_alpha);
			break;
		}
		break;
	case GR_COMBINE_LOCAL_CONSTANT:
		switch (g_gr_state.alpha_combine_local) {
		case GR_COMBINE_LOCAL_ITERATED:
			ptlv->color = g_gr_state.constant_color_value_color | ((DWORD)pv->a << 24);
			break;
		case GR_COMBINE_LOCAL_CONSTANT:
			ptlv->color = g_gr_state.constant_color_value;
			break;
		}
		break;
	}

	// set color to (1,1,1) if using texture scaled with factor 1
	// and (0,0,0) if using texture scaled with factor 0
	if(g_gr_state.color_combine_function == GR_COMBINE_FUNCTION_SCALE_OTHER ||
		g_gr_state.color_combine_function == GR_COMBINE_FUNCTION_SCALE_OTHER_ADD_LOCAL )
	{
		if(g_gr_state.color_combine_other == GR_COMBINE_OTHER_TEXTURE)
		{
			if(g_gr_state.color_combine_factor == GR_COMBINE_FACTOR_ONE)
				ptlv->color |= 0x00ffffff;
			else if(g_gr_state.color_combine_factor == GR_COMBINE_FACTOR_ZERO)
				ptlv->color &= 0xff000000;
		}
	}

	if(g_gr_state.alpha_combine_function == GR_COMBINE_FUNCTION_SCALE_OTHER &&
		g_gr_state.alpha_combine_other == GR_COMBINE_OTHER_TEXTURE)
	{
		if(g_gr_state.alpha_combine_factor == GR_COMBINE_FACTOR_ONE)
			ptlv->color |= 0xff000000;
		else if(g_gr_state.alpha_combine_factor == GR_COMBINE_FACTOR_ZERO)
		{
			ptlv->color &= 0x00ffffff;
		}
	}

		
	ptlv->tu = ((pv->tmuvtx[0].sow / pv->oow) + 0.2f) / f256;
	ptlv->tv = ((pv->tmuvtx[0].tow / pv->oow) + 0.2f) / f256;


    // scale texture coordinates according to aspect ratio
	ptlv->tu *= ah;
	ptlv->tv *= aw;

}

FX_ENTRY void FX_CALL
grDrawLine( const GrVertex *a, const GrVertex *b )
{
	VERTEX ptlv[2];
	TRACEFUNCTION("grDrawLine");

	grsubSetVertexFromGrVertex(&ptlv[0], a);
	grsubSetVertexFromGrVertex(&ptlv[1], b);


	glBegin(GL_LINES);
	glTexCoord2f(ptlv[0].tu, ptlv[0].tv);
	glColor4f((float)((ptlv[0].color&0xff0000)>>16)/256.0, (float)((ptlv[0].color&0xff00)>>8)/256.0,(float)(ptlv[0].color&0xff)/256.0, (float)((ptlv[0].color&0xff000000)>>24)/256.0);	
   	glVertex3f(ptlv[0].cord.x,ptlv[0].cord.y, ptlv[0].cord.z);
	glTexCoord2f(ptlv[1].tu, ptlv[1].tv);
	glColor4f((float)((ptlv[1].color&0xff0000)>>16)/256.0, (float)((ptlv[1].color&0xff00)>>8)/256.0,(float)(ptlv[1].color&0xff)/256.0, (float)((ptlv[1].color&0xff000000)>>24)/256.0);	
   	glVertex3f(ptlv[1].cord.x,ptlv[1].cord.y, ptlv[1].cord.z);
	glEnd();

}

FX_ENTRY void FX_CALL
grDrawPlanarPolygon( int nverts, const int ilist[], const GrVertex vlist[] )
{
	TRACEFUNCTION("grDrawPlannarPolygon");
	ERROROUT("grDrawPlannarPolygon");

	switch (g_gr_state.origin_location) {
	case GR_ORIGIN_UPPER_LEFT:
		{
	break;
		}
	case GR_ORIGIN_LOWER_LEFT:
		{
			break;
		}
	}

}

FX_ENTRY void FX_CALL
grDrawPlanarPolygonVertexList( int nverts, const GrVertex vlist[] )
{
	TRACEFUNCTION("grPlanarPolygonVertexList");

	switch (g_gr_state.origin_location) {
	case GR_ORIGIN_UPPER_LEFT:
		{
			break;
		}
	case GR_ORIGIN_LOWER_LEFT:
		{
			break;
		}
	}

}

FX_ENTRY void FX_CALL
grDrawPoint( const GrVertex *pt )
{
	TRACEFUNCTION("grDrawPoint");
	ERROROUT("grDrawPoint");

	switch (g_gr_state.origin_location) {
	case GR_ORIGIN_UPPER_LEFT:
		break;
	case GR_ORIGIN_LOWER_LEFT:
		break;
	}

}

FX_ENTRY void FX_CALL
grDrawPolygon( int nverts, const int ilist[], const GrVertex vlist[] )
{
	TRACEFUNCTION("grDrawPolygon");
	ERROROUT("grDrawPolygon");

	switch (g_gr_state.origin_location) {
	case GR_ORIGIN_UPPER_LEFT:
		{
			break;
		}
	case GR_ORIGIN_LOWER_LEFT:
		{
			break;
		}
	}

}

// Untested
FX_ENTRY void FX_CALL
grDrawPolygonVertexList( int nverts, const GrVertex vlist[] )
{
	static int i;
	static VERTEX ptlv;
	TRACEFUNCTION("grDrawPolygonVertexList");
	ERROROUT("grDrawPolygonVertexList");

	glBegin(GL_POLYGON);
	switch (g_gr_state.origin_location) {
	case GR_ORIGIN_UPPER_LEFT:
		{
			for(i = 0; i < nverts; i++)
			{
				grsubSetVertexFromGrVertex(&ptlv, &vlist[i]);
				glTexCoord2f(ptlv.tu, ptlv.tv);
				glColor4f((float)((ptlv.color&0xff0000)>>16)/255.0, (float)((ptlv.color&0xff00)>>8)/255.0,(float)(ptlv.color&0xff)/255.0, (float)((ptlv.color&0xff000000)>>24)/255.0);	
   				glVertex4f(ptlv.cord.x,ptlv.cord.y, ptlv.cord.z, ptlv.cord.w);
			}

		}
	case GR_ORIGIN_LOWER_LEFT:
		{
			for(i = nverts-1; i >= 0; i--)
			{
				grsubSetVertexFromGrVertex(&ptlv, &vlist[i]);

				glTexCoord2f(ptlv.tu, ptlv.tv);
				glColor4f((float)((ptlv.color&0xff0000)>>16)/255.0, (float)((ptlv.color&0xff00)>>8)/255.0,(float)(ptlv.color&0xff)/255.0, (float)((ptlv.color&0xff000000)>>24)/255.0);	
   				glVertex4f(ptlv.cord.x,ptlv.cord.y, ptlv.cord.z, ptlv.cord.w);
			}
			break;
		}
	}
	glEnd();

}

FX_ENTRY void FX_CALL
grDrawTriangle( const GrVertex *a, const GrVertex *b, const GrVertex *c )
{
	static VERTEX ptlv[3];
	static BOOL glblendon;
	TRACEFUNCTION("grDrawTriangle");

	switch (g_gr_state.origin_location) {
	case GR_ORIGIN_UPPER_LEFT:
		{
			grsubSetVertexFromGrVertex(&ptlv[0], a);
			grsubSetVertexFromGrVertex(&ptlv[1], b);
			grsubSetVertexFromGrVertex(&ptlv[2], c);
			break;
		}
	case GR_ORIGIN_LOWER_LEFT:
		{
			grsubSetVertexFromGrVertex(&ptlv[0], c);
			grsubSetVertexFromGrVertex(&ptlv[1], b);
			grsubSetVertexFromGrVertex(&ptlv[2], a);
			break;
		}
	}

	// Draw

	// 2 passes needed to acomplish this method 
	if(g_gr_state.color_combine_function == GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL_ADD_LOCAL &&
		g_gr_state.color_combine_factor == GR_COMBINE_FACTOR_TEXTURE_ALPHA &&
		g_gr_state.color_combine_local == GR_COMBINE_LOCAL_ITERATED &&
		g_gr_state.color_combine_other == GR_COMBINE_OTHER_TEXTURE)
	{
		glDisable(GL_TEXTURE_2D);
		glDepthMask(FALSE);

		glBegin(GL_TRIANGLES);
		glColor4f((float)((ptlv[0].color&0xff0000)>>16)/255.0, (float)((ptlv[0].color&0xff00)>>8)/255.0,(float)(ptlv[0].color&0xff)/255.0, (float)((ptlv[0].color&0xff000000)>>24)/255.0);	
   		glVertex4f(ptlv[0].cord.x,ptlv[0].cord.y, ptlv[0].cord.z, ptlv[0].cord.w);

		glColor4f((float)((ptlv[1].color&0xff0000)>>16)/255.0, (float)((ptlv[1].color&0xff00)>>8)/255.0,(float)(ptlv[1].color&0xff)/255.0, (float)((ptlv[1].color&0xff000000)>>24)/255.0);	
   		glVertex4f(ptlv[1].cord.x,ptlv[1].cord.y, ptlv[1].cord.z, ptlv[1].cord.w);

		glColor4f((float)((ptlv[2].color&0xff0000)>>16)/255.0, (float)((ptlv[2].color&0xff00)>>8)/255.0,(float)(ptlv[2].color&0xff)/255.0, (float)((ptlv[2].color&0xff000000)>>24)/255.0);	
   		glVertex4f(ptlv[2].cord.x,ptlv[2].cord.y, ptlv[2].cord.z, ptlv[2].cord.w);
		glEnd();

		glEnable(GL_TEXTURE_2D);
		glDepthMask(g_gr_state.depth_mask);
		glblendon = glIsEnabled(GL_BLEND);
		glEnable(GL_BLEND);

		glBegin(GL_TRIANGLES);
		glTexCoord2f(ptlv[0].tu, ptlv[0].tv);
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);	
   		glVertex4f(ptlv[0].cord.x,ptlv[0].cord.y, ptlv[0].cord.z, ptlv[0].cord.w);

		glTexCoord2f(ptlv[1].tu, ptlv[1].tv);
   		glVertex4f(ptlv[1].cord.x,ptlv[1].cord.y, ptlv[1].cord.z, ptlv[1].cord.w);

		glTexCoord2f(ptlv[2].tu, ptlv[2].tv);	
   		glVertex4f(ptlv[2].cord.x,ptlv[2].cord.y, ptlv[2].cord.z, ptlv[2].cord.w);
		glEnd();

		if(!glblendon)
			glDisable(GL_BLEND);


	}
	else {
		glBegin(GL_TRIANGLES);
		glTexCoord2f(ptlv[0].tu, ptlv[0].tv);
		glColor4f((float)((ptlv[0].color&0xff0000)>>16)/255.0, (float)((ptlv[0].color&0xff00)>>8)/255.0,(float)(ptlv[0].color&0xff)/255.0, (float)((ptlv[0].color&0xff000000)>>24)/255.0);	
   		glVertex4f(ptlv[0].cord.x,ptlv[0].cord.y, ptlv[0].cord.z, ptlv[0].cord.w);

		glTexCoord2f(ptlv[1].tu, ptlv[1].tv);
		glColor4f((float)((ptlv[1].color&0xff0000)>>16)/255.0, (float)((ptlv[1].color&0xff00)>>8)/255.0,(float)(ptlv[1].color&0xff)/255.0, (float)((ptlv[1].color&0xff000000)>>24)/255.0);	
   		glVertex4f(ptlv[1].cord.x,ptlv[1].cord.y, ptlv[1].cord.z, ptlv[1].cord.w);

		glTexCoord2f(ptlv[2].tu, ptlv[2].tv);	
		glColor4f((float)((ptlv[2].color&0xff0000)>>16)/255.0, (float)((ptlv[2].color&0xff00)>>8)/255.0,(float)(ptlv[2].color&0xff)/255.0, (float)((ptlv[2].color&0xff000000)>>24)/255.0);	
   		glVertex4f(ptlv[2].cord.x,ptlv[2].cord.y, ptlv[2].cord.z, ptlv[2].cord.w);
		glEnd();
	}
}

FX_ENTRY void FX_CALL
guAADrawTriangleWithClip( const GrVertex *a, const GrVertex
                         *b, const GrVertex *c)
{
	TRACEFUNCTION("guAADrawTriangleWithClip");

	GLID3D_ANTIALIAS_ENABLE
	guDrawTriangleWithClip(a, b, c);
	GLID3D_ANTIALIAS_DISABLE
}

FX_ENTRY void FX_CALL
guDrawTriangleWithClip(
                       const GrVertex *a,
                       const GrVertex *b,
                       const GrVertex *c
                       )
{
	grDrawTriangle(a,b,c);
}

FX_ENTRY void FX_CALL
guDrawPolygonVertexListWithClip( int nverts, const GrVertex vlist[] )
{
	TRACEFUNCTION("guDrawPolygonVertexListWithClip");
	ERROROUT("guDrawPolygonVertexListWithClip");

	grDrawPolygonVertexList(nverts, vlist);
}

/////////////////////////////////////////////////////////////////////////////
