/////////////////////////////////////////////////////////////////////////////
// GliD3D_texture.cpp : No comment
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Directive

#include "StdHdr.h"
#pragma hdrstop
#include "GliD3D.h"
/////////////////////////////////////////////////////////////////////////////
// Defines / Variables

typedef struct {
	DDPIXELFORMAT	ddpf;
	CHAR			pszName[256];
	INT				nBits;
	INT				nR;
	INT				nG;
	INT				nB;
	INT				nA;
	BOOL			bPalette;
} *LPGB3DCORETEXTUREFORMATINFO, GB3DCORETEXTUREFORMATINFO;

/////////////////////////////////////////////////////////////////////////////
// Prototypes

VOID GB3DCoreGetTextureFormatInfo(LPGB3DCORETEXTUREFORMATINFO pInfo, INT nIndex);

/////////////////////////////////////////////////////////////////////////////
// Implements

FX_ENTRY FxU32 FX_CALL 
grTexCalcMemRequired(
                     GrLOD_t lodmin, GrLOD_t lodmax,
                     GrAspectRatio_t aspect, GrTextureFormat_t fmt)
{
	TRACEFUNCTION("grTexCalcMemRequired");

	return GLID3D_TEXCALCMEMREQUIRED;
}

FX_ENTRY FxU32 FX_CALL 
grTexTextureMemRequired( FxU32     evenOdd,
                                 GrTexInfo *info   )
{
	TRACEFUNCTION("grTexTextureMemRequired");

	return GLID3D_TEXTEXTUREMEMREQUIRED;
}

FX_ENTRY FxU32 FX_CALL 
grTexMinAddress( GrChipID_t tmu )
{
	TRACEFUNCTION("grTexMinAddress");

	return GLID3D_TEXMINADDRESS;
}

FX_ENTRY FxU32 FX_CALL 
grTexMaxAddress( GrChipID_t tmu )
{
	TRACEFUNCTION("grTexMaxAddress");

	return GLID3D_TEXMAXADDRESS;
}

FX_ENTRY void FX_CALL 
grTexNCCTable( GrChipID_t tmu, GrNCCTable_t table )
{
	TRACEFUNCTION("grTexNCCTable");

//	WARNING("Not supported yet");
}

FX_ENTRY void FX_CALL 
grTexSource( GrChipID_t tmu,
             FxU32      startAddress,
             FxU32      evenOdd,
             GrTexInfo  *info )
{
	TRACEFUNCTION("grTexSource");

	if (startAddress < 0 || startAddress >= GLID3D_TEXMAXADDRESS) {
		WARNING("Wrong source");
		return;
	}

	
	INT nTmu = grsubGetTmuIndex(tmu);
	glBindTexture(GL_TEXTURE_2D, g_gr_tmu[nTmu].texture[startAddress].texId);
	grTexClampMode(nTmu, g_gr_tmu[nTmu].s_clampmode, g_gr_tmu[nTmu].t_clampmode);
	g_gr_tmu[nTmu].texture_source_start_address = startAddress;
	g_gr_tmu[nTmu].texture_source_even_odd = evenOdd;
	g_gr_tmu[nTmu].texture_source_info = *info;

	grTexMipMapMode(tmu, g_gr_tmu[nTmu].mipmap_mode, FALSE);
	grMegaCombineEverythingSetup();

}

FX_ENTRY void FX_CALL 
grTexClampMode(
               GrChipID_t tmu,
               GrTextureClampMode_t s_clampmode,
               GrTextureClampMode_t t_clampmode
               )
{
	TRACEFUNCTION("grTexClampMode");

	INT nTmu = grsubGetTmuIndex(tmu);
	g_gr_tmu[nTmu].s_clampmode = s_clampmode;
	g_gr_tmu[nTmu].t_clampmode = t_clampmode;

	if(s_clampmode == GR_TEXTURECLAMP_WRAP)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	else
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);

	if(t_clampmode == GR_TEXTURECLAMP_WRAP)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	else
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

//	g_gr_pd3ddevice->SetTextureStageState(nTmu, D3DTSS_ADDRESSU, s_clampmode == GR_TEXTURECLAMP_WRAP ? D3DTADDRESS_WRAP : D3DTADDRESS_CLAMP);
//	g_gr_pd3ddevice->SetTextureStageState(nTmu, D3DTSS_ADDRESSV, t_clampmode == GR_TEXTURECLAMP_WRAP ? D3DTADDRESS_WRAP : D3DTADDRESS_CLAMP);
}

FX_ENTRY void FX_CALL 
grTexCombine(
             GrChipID_t tmu,
             GrCombineFunction_t rgb_function,
             GrCombineFactor_t rgb_factor, 
             GrCombineFunction_t alpha_function,
             GrCombineFactor_t alpha_factor,
             FxBool rgb_invert,
             FxBool alpha_invert
             )
{
	TRACEFUNCTION("grTexCombine");

	INT nTmu = grsubGetTmuIndex(tmu);
	g_gr_tmu[nTmu].texture_combine_rgb_function = rgb_function;
	g_gr_tmu[nTmu].texture_combine_rgb_factor = rgb_factor;
	g_gr_tmu[nTmu].texture_combine_alpha_function = alpha_function;
	g_gr_tmu[nTmu].texture_combine_alpha_factor = alpha_factor;
}

FX_ENTRY void FX_CALL 
grTexCombineFunction(
                     GrChipID_t tmu,
                     GrTextureCombineFnc_t fnc
                     )
{
	TRACEFUNCTION("grTexCombineFunction");

	switch (fnc) {
	case GR_TEXTURECOMBINE_ZERO          : break;
	case GR_TEXTURECOMBINE_DECAL         : break;
	case GR_TEXTURECOMBINE_OTHER         : break;
	case GR_TEXTURECOMBINE_ADD           : break;
	case GR_TEXTURECOMBINE_MULTIPLY      : break;
	case GR_TEXTURECOMBINE_SUBTRACT      : break;
	case GR_TEXTURECOMBINE_DETAIL        : break;
	case GR_TEXTURECOMBINE_DETAIL_OTHER  : break;
	case GR_TEXTURECOMBINE_TRILINEAR_ODD : break;
	case GR_TEXTURECOMBINE_TRILINEAR_EVEN: break;
	case GR_TEXTURECOMBINE_ONE           : break;
	}
}

FX_ENTRY void FX_CALL 
grTexDetailControl(
                   GrChipID_t tmu,
                   int lod_bias,
                   FxU8 detail_scale,
                   float detail_max
                   )
{
	TRACEFUNCTION("grTexDetailControl");

//	WARNING("Not supported yet");
}

FX_ENTRY void FX_CALL 
grTexFilterMode(
                GrChipID_t tmu,
                GrTextureFilterMode_t minfilter_mode,
                GrTextureFilterMode_t magfilter_mode
                )
{
	TRACEFUNCTION("grTexFilterMode");
	INT nTmu = grsubGetTmuIndex(tmu);
	g_gr_tmu[nTmu].texture_minfilter_mode = minfilter_mode;
	g_gr_tmu[nTmu].texture_magfilter_mode = magfilter_mode;
	grTexMipMapMode(tmu, g_gr_tmu[nTmu].mipmap_mode, FALSE);

}


FX_ENTRY void FX_CALL 
grTexLodBiasValue(GrChipID_t tmu, float bias )
{
	TRACEFUNCTION("grTexLodBiasValue");

//	WARNING("Not supported yet");
}

#define GL2IDE_FORCE_MIPMAPS 0
FX_ENTRY void FX_CALL 
grTexDownloadMipMap( GrChipID_t tmu,
                     FxU32      startAddress,
                     FxU32      evenOdd,
                     GrTexInfo  *info )
{
	int aw, ah, w,h, lod, size=0,i;
	unsigned char *dataptr;

	TRACEFUNCTION("grTexDownloadMipMap");
	grsubAspectToInt(&aw, &ah, info->aspectRatio);
if(!g_gr_bForceMipMaps) {
		switch (info->format) {
			case GR_TEXFMT_ALPHA_INTENSITY_88:
				size=2;
			case GR_TEXFMT_AP_88:
				size=2;
			case GR_TEXFMT_ALPHA_INTENSITY_44:
			case GR_TEXFMT_INTENSITY_8:
			case GR_TEXFMT_P_8:
			case GR_TEXFMT_ALPHA_8            :
			case GR_TEXFMT_YIQ_422:
			case GR_TEXFMT_RGB_332:
			size=1;
			break;
			case GR_TEXFMT_AYIQ_8422:
			case GR_TEXFMT_ARGB_8332:
			case GR_TEXFMT_RGB_565:
			case GR_TEXFMT_ARGB_1555:
			case GR_TEXFMT_ARGB_4444:
			size=2;
			break;

			default:
	//			error << "cp format" << endl;
				return;
			break;
		}

		dataptr=(unsigned char *)info->data;
		switch(evenOdd) {
		case GR_MIPMAPLEVELMASK_BOTH:
			for(i=info->largeLod; i>=info->smallLod; i--) {
				w=grsubLodToInt(i)/ah;
				h=grsubLodToInt(i)/aw;
				grTexDownloadMipMapLevel( tmu, startAddress, i, info->largeLod,
					info->aspectRatio, info->format, evenOdd, dataptr);
				dataptr+=(w*h*size);
			}
			break;
		case GR_MIPMAPLEVELMASK_EVEN:
			for(i=info->largeLod; i>=info->smallLod; i--) {
				if((i%2)==0) {
					w=grsubLodToInt(i)/ah;
					h=grsubLodToInt(i)/aw;
					grTexDownloadMipMapLevel( tmu, startAddress, i, info->largeLod,
					info->aspectRatio, info->format, evenOdd, dataptr);
					dataptr+=(w*h*size);
				}
			}

			break;
		case GR_MIPMAPLEVELMASK_ODD:		
			for(i=info->largeLod; i>=info->smallLod; i--) {
				if((i%2)==1) {
					w=grsubLodToInt(i)/ah;
					h=grsubLodToInt(i)/aw;
					grTexDownloadMipMapLevel( tmu, startAddress, i, info->largeLod,
					info->aspectRatio, info->format, evenOdd, dataptr);
					dataptr+=(w*h*size);
				}
			}

			break;
		default:
			//error
			break;
		}
	}
	else
	grTexDownloadMipMapLevel( tmu, startAddress, info->largeLod, info->largeLod,
           info->aspectRatio, info->format, evenOdd, info->data );

}

FX_ENTRY void FX_CALL 
grTexDownloadMipMapLevel( GrChipID_t        tmu,
                          FxU32             startAddress,
                          GrLOD_t           thisLod,
                          GrLOD_t           largeLod,
                          GrAspectRatio_t   aspectRatio,
                          GrTextureFormat_t format,
                          FxU32             evenOdd,
                          void              *data )
{
	int w,h,aw, ah,x, y, i , q, tlod;
	GLenum err;
	char tmp[2024];
	static unsigned short *src;
	static unsigned char *tbuffer, *dest, *src8;


	TRACEFUNCTION("grTexDownloadMipMapLevel");
//	ofstream error("error.txt",ios::out | ios::app, 0); 


	int nTmu = grsubGetTmuIndex(tmu);
	LPGLID3D_TMU ptmu = &g_gr_tmu[nTmu];
	LPGLID3D_TEXTURE ptex = &ptmu->texture[startAddress];	  

	if(ptex->texId == -1) {
		//genereate a new texname
		glGenTextures(1, &ptex->texId);	
	}

	if(tbuffer == NULL)
		tbuffer= (unsigned char *)malloc(256*256*4);

	//create new texture object
	glBindTexture(GL_TEXTURE_2D, ptex->texId);

	grsubAspectToInt(&aw, &ah, aspectRatio);

	//Get size of lod
	w=grsubLodToInt(thisLod)/ah;
	h=grsubLodToInt(thisLod)/aw;

	switch(evenOdd) {
	case GR_MIPMAPLEVELMASK_BOTH:
		tlod =(thisLod - largeLod);
		break;;
	case GR_MIPMAPLEVELMASK_EVEN:
		tlod=((thisLod  -largeLod)/2)-1;
		break;
	case GR_MIPMAPLEVELMASK_ODD:		
		tlod=(thisLod -1 - largeLod)/2;
		break;
	default:
		//error
		break;
	}

	switch (format) {
		case GR_TEXFMT_INTENSITY_8:
//		error << "oimplemented format: Intensity 8" << endl;
		break;
		case GR_TEXFMT_ALPHA_INTENSITY_88:
//		error << "oimplemented format: AI 8" << endl;
		break;
		case GR_TEXFMT_AP_88:
		// not implemented/
//		error << "oimplemented format: AP 8" << endl;
		break;
		case GR_TEXFMT_ALPHA_INTENSITY_44:
		case GR_TEXFMT_P_8:
			
//		error << "oimplemented format" << endl;
		

		break;
		case GR_TEXFMT_ALPHA_8            :
//		glTexImage2D(GL_TEXTURE_2D, tlod, GL_ALPHA, w, h, 0, GL_ALPHA, GL_UNSIGNED_BYTE, data);
//		error << "ALPHA 8" << endl;
		break;

		case GR_TEXFMT_AYIQ_8422:
			for(x = 0; x < w*h; x++)
			{
				src = (unsigned short *)(data)+x; dest = (unsigned char*)tbuffer +x*4;
				y = (((*src) >> 4 ) & 0x0f);
				i = (((*src) >> 2 ) & 0x03);
				q = ((*src)  & 0x03);
				*(dest)   = y + 0.95*i + 0.62*q;
				*(dest+1) = y - 0.28*i - 0.64*q;
				*(dest+2) = y - 1.11*i + 1.73*q;
				*(dest+3) = (((*src) >> 8 ) & 0xff);
			}
//			glTexImage2D(GL_TEXTURE_2D, tlod, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tbuffer);
//			error << "AYIQ_8422 " << w <<" " << h << endl;
			break;

		case GR_TEXFMT_YIQ_422:
			for(x = 0; x < w*h; x++)
			{
				src8 = (unsigned char *)(data)+x; dest = (unsigned char*)tbuffer +x*4;
				y = (((*src8) >> 4 ) & 0x0f);
				i = (((*src8) >> 2 ) & 0x03);
				q = (((*src8) >> 0 )  & 0x03);
				*(dest)   = y + 0.95*i + 0.62*q;
				*(dest+1) = y - 0.28*i - 0.64*q;
				*(dest+2) = y - 1.11*i + 1.73*q;
				*(dest+3) = 0;
			}
//			glTexImage2D(GL_TEXTURE_2D, tlod, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tbuffer);
//			error << "YIQ_422 " << w <<" " << h << endl;
			break;

		

		case GR_TEXFMT_RGB_332:
			// Untested 
			for(x=0; x<w*h; x++) {
				src = (unsigned short *)(data)+x; dest = (unsigned char*)tbuffer +x*4;
				*(dest)   = ((*src ) & (0x007<<5))  + (((*src >> 5 ) & 0x007) << 2)+ (((*src >> 5 ) & 0x007) >> 1);
				*(dest+1) = ((*src << 3) & (0x007<<5))  + (((*src >> 2 ) & 0x007) << 2)+ (((*src >> 2 ) & 0x007) >> 1);
				*(dest+2) = (((*src ) & 0x003)<<6) + (((*src ) & 0x003) << 4)+ (((*src ) & 0x003) << 4)+ (((*src ) & 0x003));
				*(dest+3) = 0xff;
			}

//			glTexImage2D(GL_TEXTURE_2D, tlod, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tbuffer);
		break;

		case GR_TEXFMT_ARGB_8332:
			// Untested 
			for(x=0; x<w*h; x++) {
				src = (unsigned short *)(data)+x; dest = (unsigned char*)tbuffer +x*4;
				*(dest)   = ((*src ) & (0x007<<5))  + (((*src >> 5 ) & 0x007) << 2)+ (((*src >> 5 ) & 0x007) >> 1);
				*(dest+1) = ((*src << 3) & (0x007<<5))  + (((*src >> 2 ) & 0x007) << 2)+ (((*src >> 2 ) & 0x007) >> 1);
				*(dest+2) = (((*src ) & 0x003)<<6) + (((*src ) & 0x003) << 4)+ (((*src ) & 0x003) << 4)+ (((*src ) & 0x003));
				*(dest+3) = (((*src) >> 8 ) & 0xff);
			}

//			glTexImage2D(GL_TEXTURE_2D, tlod, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tbuffer);
		break;

		case GR_TEXFMT_RGB_565:
			for(x=0; x<w*h; x++) {
				src = (unsigned short *)(data)+x; dest = (unsigned char*)tbuffer +x*4;
				*(dest)   = (((*src >> 11 ) & 0x001f)<<3)  + (((*src >> 11 ) & 0x001f) >> 2);
				*(dest+1) = (((*src >> 5 ) & 0x003f)<<2) + (((*src >> 5 ) & 0x003f) >> 4);
				*(dest+2) = (((*src >> 0 ) & 0x001f)<<3) + (((*src >> 0 ) & 0x001f) >> 2);
				*(dest+3) = 0xff;
			}

//			glTexImage2D(GL_TEXTURE_2D, tlod, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tbuffer);
//			error << "RGB_565" << w <<" " << h << endl;
			break;
			
		case GR_TEXFMT_ARGB_1555:
			for(x=0; x<w*h; x++) {
				src = (unsigned short *)(data)+x; dest = (unsigned char*)tbuffer +x*4;
				*(dest)   = (((*src >> 10 ) & 0x001f)<<3) + (((*src >> 10 ) & 0x001f) >> 2);
				*(dest+1) = (((*src >> 5 ) & 0x001f)<<3) + (((*src >> 5 ) & 0x001f) >> 2);
				*(dest+2) = (((*src >> 0 ) & 0x001f)<<3) + (((*src >> 0 ) & 0x001f) >> 2);
				*(dest+3) =0 - ((*src >> 15) & 0x0001);

			}
//			glTexImage2D(GL_TEXTURE_2D, tlod, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tbuffer);
//			error << "RGB_555" << w <<" " << h << endl;
			break;

		case GR_TEXFMT_ARGB_4444:
			for(x=0; x<w*h; x++) {
				src = (unsigned short *)(data)+x; dest = (unsigned char*)tbuffer +x*4;
				*(dest)   = (((*src >> 8 ) & 0x000f)<<4)+((*src >> 8) & 0xf );
				*(dest+1) = (((*src >> 4 ) & 0x000f)<<4)+((*src >> 4) & 0xf );
				*(dest+2) = (((*src >> 0 ) & 0x000f)<<4)+((*src >> 0) & 0xf );
				*(dest+3) = (((*src >> 12) & 0x000f)<<4)+((*src >> 12)& 0xf );
			}
			
//			glTexImage2D(GL_TEXTURE_2D, tlod, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tbuffer);
//			error << "RGB_4444" << w <<" " << h << endl;
			break;

		default:
//			error << "cp format" << endl;
		break;
	}
	if(g_gr_bForceMipMaps) {
		gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, w, h, GL_RGBA, GL_UNSIGNED_BYTE, tbuffer);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	}
	else{
		glTexImage2D(GL_TEXTURE_2D, tlod, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tbuffer);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	}

	ptex->info.smallLod = thisLod;
	ptex->info.largeLod = largeLod;
	ptex->info.aspectRatio = aspectRatio;
	ptex->info.format = format;
	ptex->info.data = data;
}

FX_ENTRY void FX_CALL 
grTexDownloadMipMapLevelPartial( GrChipID_t        tmu,
                                 FxU32             startAddress,
                                 GrLOD_t           thisLod,
                                 GrLOD_t           largeLod,
                                 GrAspectRatio_t   aspectRatio,
                                 GrTextureFormat_t format,
                                 FxU32             evenOdd,
                                 void              *data,
                                 int               start,
                                 int               end )
{
	TRACEFUNCTION("grTexDownloadMipMapLevelPartial");

//	WARNING("Not supported yet");
}


FX_ENTRY void FX_CALL 
ConvertAndDownloadRle( GrChipID_t        tmu,
                        FxU32             startAddress,
                        GrLOD_t           thisLod,
                        GrLOD_t           largeLod,
                        GrAspectRatio_t   aspectRatio,
                        GrTextureFormat_t format,
                        FxU32             evenOdd,
                        FxU8              *bm_data,
                        long              bm_h,
                        FxU32             u0,
                        FxU32             v0,
                        FxU32             width,
                        FxU32             height,
                        FxU32             dest_width,
                        FxU32             dest_height,
                        FxU16             *tlut)
{
	TRACEFUNCTION("ConvertAndDownloadRle");

//	WARNING("Not supported yet");
}

FX_ENTRY void FX_CALL 
grCheckForRoom(FxI32 n)
{
	TRACEFUNCTION("grCheckForRoom");

//	WARNING("Not supported yet");
}

FX_ENTRY void FX_CALL
grTexDownloadTable( GrChipID_t   tmu,
                    GrTexTable_t type, 
                    void         *data )
{
	TRACEFUNCTION("grTexDownloadTable");

//	WARNING("Not supported yet");
}

FX_ENTRY void FX_CALL
grTexDownloadTablePartial( GrChipID_t   tmu,
                           GrTexTable_t type, 
                           void         *data,
                           int          start,
                           int          end )
{
	TRACEFUNCTION("grTexDownloadTablePartial");

//	WARNING("Not supported yet");
}

FX_ENTRY void FX_CALL 
grTexMipMapMode( GrChipID_t     tmu, 
                 GrMipMapMode_t mode,
                 FxBool         lodBlend )
{
	TRACEFUNCTION("grTexMipMapMode");

	INT nTmu = grsubGetTmuIndex(tmu);
	g_gr_tmu[nTmu].mipmap_mode = mode;

	if(g_gr_tmu[nTmu].texture_magfilter_mode == GR_TEXTUREFILTER_BILINEAR)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	else
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);


	if(g_gr_bForceMipMaps) {
		if(g_gr_tmu[nTmu].texture_minfilter_mode == GR_TEXTUREFILTER_BILINEAR)
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		else
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
	}
	else {
		switch(mode) {

		case GR_MIPMAP_DISABLE:
			if(g_gr_tmu[nTmu].texture_minfilter_mode == GR_TEXTUREFILTER_BILINEAR)
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			else
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
			break;
		case GR_MIPMAP_NEAREST:
			if(g_gr_tmu[nTmu].texture_minfilter_mode == GR_TEXTUREFILTER_BILINEAR)
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
			else
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST);
			break;

		case GR_MIPMAP_NEAREST_DITHER:
			if(g_gr_tmu[nTmu].texture_minfilter_mode == GR_TEXTUREFILTER_BILINEAR)
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
			else
				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
			break;

		default:
			break;
		}
	}
}

FX_ENTRY void FX_CALL 
grTexMultibase( GrChipID_t tmu,
                FxBool     enable )
{
	TRACEFUNCTION("grTexMultibase");

//	WARNING("Not supported yet");
}

FX_ENTRY void FX_CALL
grTexMultibaseAddress( GrChipID_t       tmu,
                       GrTexBaseRange_t range,
                       FxU32            startAddress,
                       FxU32            evenOdd,
                       GrTexInfo        *info )
{
	TRACEFUNCTION("grTexMultibaseAddress");

//	WARNING("Not supported yet");
}

/////////////////////////////////////////////////////////////////////////////

